# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/metrics.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import net_pb2 as harness_dot_net__pb2
from harness import wire_pb2 as harness_dot_wire__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/metrics.proto',
  package='harness.metrics',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x15harness/metrics.proto\x12\x0fharness.metrics\x1a\x11harness/net.proto\x1a\x12harness/wire.proto\"F\n\nPrometheus\x12\x0e\n\x06prefix\x18\x01 \x01(\t\x12(\n\x04\x62ind\x18\x02 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x01\x62\x06proto3'
  ,
  dependencies=[harness_dot_net__pb2.DESCRIPTOR,harness_dot_wire__pb2.DESCRIPTOR,])




_PROMETHEUS = _descriptor.Descriptor(
  name='Prometheus',
  full_name='harness.metrics.Prometheus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='prefix', full_name='harness.metrics.Prometheus.prefix', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bind', full_name='harness.metrics.Prometheus.bind', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=151,
)

_PROMETHEUS.fields_by_name['bind'].message_type = harness_dot_net__pb2._SOCKET
DESCRIPTOR.message_types_by_name['Prometheus'] = _PROMETHEUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Prometheus = _reflection.GeneratedProtocolMessageType('Prometheus', (_message.Message,), {
  'DESCRIPTOR' : _PROMETHEUS,
  '__module__' : 'harness.metrics_pb2'
  # @@protoc_insertion_point(class_scope:harness.metrics.Prometheus)
  })
_sym_db.RegisterMessage(Prometheus)


_PROMETHEUS.fields_by_name['bind']._options = None
# @@protoc_insertion_point(module_scope)
