syntax = "proto3";

package harness.grpc;

import "harness/net.proto";
import "harness/wire.proto";

message Channel {
    harness.net.Socket address = 1 [(harness.mark).protocol = GRPC];
}

message Server {
    harness.net.Socket bind = 1 [(harness.mark).protocol = GRPC];
}
