syntax = "proto3";

package harness.redis;

import "harness/wire.proto";
import "harness/net.proto";

message Connection {
    harness.net.Socket address = 1 [(harness.mark).protocol = TCP];
    int32 db = 2;
}
