syntax = "proto3";

package harness.postgres;

import "harness/wire.proto";
import "harness/net.proto";

message Pool {
    harness.net.Socket address = 1 [(harness.mark).protocol = TCP];
    string username = 2;
    string password = 3;
    string database = 4;
    uint32 min_size = 5;
    uint32 max_size = 6;
}
